import os
import json
from datetime import datetime
from pathlib import Path
import hashlib

    
def find_project_root(start_dir=None):
    current = Path(start_dir or Path.cwd()).resolve()
    for parent in [current] + list(current.parents):
        if (parent / ".git").exists() or (parent / "pyproject.toml").exists() or (parent / "setup.py").exists():
            return parent
    return current  # fallback


PROJECT_ROOT = find_project_root()
LOG_DIR = PROJECT_ROOT / ".llm_logger" / "logs"
LOG_DIR.mkdir(parents=True, exist_ok=True)


# === Serialization ===

def extract_json(obj):
    if hasattr(obj, "model_dump"):
        obj = obj.model_dump()
    elif hasattr(obj, "dict"):
        obj = obj.dict()
    try:
        return json.loads(json.dumps(obj, default=str))
    except Exception:
        return {"raw": repr(obj)}

# === Load Hashes ===
def normalize_messages(messages):
    return [{"role": m["role"], "content": m["content"]} for m in messages]

def hash_messages(messages):
    normalized = normalize_messages(messages)
    string = json.dumps(normalized, sort_keys=True, ensure_ascii=False)
    return hashlib.sha256(string.encode("utf-8")).hexdigest()[:12]

    
def session_file_exists(messages):
    session_id = hash_messages(messages)
    return (LOG_DIR / f"{session_id}.json").exists()

def log_call(*, provider, args, kwargs, response, request_start_timestamp, request_end_timestamp, logging_account_id):
    messages = kwargs.get("messages", [])
    session_id = kwargs.get("session_id", None)

    log_entry = {
        "start_time": request_start_timestamp,
        "end_time": request_end_timestamp,
        "request_body": {
            "args": repr(args),  # to avoid serialization errors
            "kwargs": extract_json(kwargs),
        },
        "response": extract_json(response),
        "provider": provider,
        "logging_account_id": logging_account_id,
    }

    # If no session_id is provided, try to match based on message history
    if not session_id:
        # Use Dynamic Session IDs
        #Walk backwards through messages to find if any existing log files are available
        old_session_id = None
        for i in range(1, len(messages)):
            message_subset = messages[0:-i]
            if session_file_exists(message_subset):
                old_session_id = hash_messages(message_subset)
                break
        logs = []
        if old_session_id:
            #TODO: open the file at the path LOG_DIR / f"{old_session_id}.json"
            old_path = LOG_DIR / f"{old_session_id}.json"
            try:
                with open(old_path, "r") as f:
                    logs = json.load(f)
                    if not isinstance(logs, list):
                        logs = []
                #remove the old file since were going to create a new one
                os.remove(old_path)
            except Exception as e:
                print(f"Warning reading existing log: {e}")
                logs = []


        logs.append(log_entry)
        new_session_id = hash_messages(messages)
        new_path = LOG_DIR / f"{new_session_id}.json"
        with open(new_path, "w") as f:
                json.dump(logs, f, indent=2)
    else:
        # Use the provided session_id
        filepath = LOG_DIR / f"{session_id}.json"
        if filepath.exists():
            with open(filepath, "r") as f:
                logs = json.load(f)
                if not isinstance(logs, list):
                    logs = []
        else:
            logs = []

        logs.append(log_entry)
        with open(filepath, "w") as f:
            json.dump(logs, f, indent=2)





import os
import json

DEFAULT_CONFIG = {
    "log_dir": ".llm_logger/logs",
    "cloud": False,
    "logging_account_id": None
}

CONFIG_PATH = os.path.expanduser("~/.llm_logger/config.json")

def load_config():
    if not os.path.exists(CONFIG_PATH):
        os.makedirs(os.path.dirname(CONFIG_PATH), exist_ok=True)
        with open(CONFIG_PATH, "w") as f:
            json.dump(DEFAULT_CONFIG, f, indent=2)
        return DEFAULT_CONFIG

    try:
        with open(CONFIG_PATH, "r") as f:
            config = json.load(f)
        return {**DEFAULT_CONFIG, **config}
    except Exception:
        return DEFAULT_CONFIG

config = load_config()



from fastapi import FastAPI, Request
from fastapi.staticfiles import StaticFiles
from fastapi.responses import FileResponse, JSONResponse, HTMLResponse
from fastapi.templating import Jinja2Templates
from pathlib import Path
import json

def find_project_root(start_dir=None):
    current = Path(start_dir or Path.cwd()).resolve()
    for parent in [current] + list(current.parents):
        if (parent / ".git").exists() or (parent / "pyproject.toml").exists() or (parent / "setup.py").exists():
            return parent
    return current  # fallback


PROJECT_ROOT = find_project_root()
LOG_DIR = PROJECT_ROOT / ".llm_logger" / "logs"
LOG_DIR.mkdir(parents=True, exist_ok=True)

BASE_DIR = Path(__file__).parent
STATIC_DIR = BASE_DIR / "static"

def create_app(base_url=""):
    """Create the FastAPI app with an optional base_url parameter."""
    app = FastAPI()
    app.mount(f"{base_url}/static", StaticFiles(directory=STATIC_DIR, html=True), name="static")

    # Store the base_url in the app state
    app.state.base_url = base_url

    @app.get(f"{base_url}/")
    def index(request: Request):
        # Instead of returning the file directly, we'll inject the base_url
        with open(STATIC_DIR / "index.html", "r") as f:
            content = f.read()
        
        # Inject the base_url as a JavaScript variable
        script_tag = f'<script>window.BASE_URL = "{app.state.base_url}";</script>'
        modified_content = content.replace('</head>', f'{script_tag}</head>')
        
        return HTMLResponse(content=modified_content)

    @app.get(f"{base_url}/sessions/{{session_id}}", response_class=HTMLResponse)
    def serve_session_view(request: Request, session_id: str):
        # Inject the base_url into session.html too
        with open(STATIC_DIR / "session.html", "r") as f:
            content = f.read()
        
        script_tag = f'<script>window.BASE_URL = "{app.state.base_url}";</script>'
        modified_content = content.replace('</head>', f'{script_tag}</head>')
        
        return HTMLResponse(content=modified_content)

    @app.get(f"{base_url}/api/sessions")
    def list_sessions():
        files = sorted(LOG_DIR.glob("*.json"))
        return [f.name for f in files]

    @app.get(f"{base_url}/api/sessions/{{session_id}}")
    def get_session(session_id: str):
        path = LOG_DIR / f"{session_id}.json"
        if not path.exists():
            return JSONResponse(status_code=404, content={"error": "Not found"})
        with open(path) as f:
            return json.load(f)
            
    return app

# Create a default app instance for backward compatibility
app = create_app()
